@@ -2094,7 +2094,9 @@ save_account_details_now (GdmSessionWorker *worker)
 }
 
 static void
-on_settings_is_loaded_changed (GdmSessionWorker *worker)
+on_settings_is_loaded_changed (GdmSessionSettings *user_settings,
+                               GParamSpec         *pspec,
+                               GdmSessionWorker   *worker)
 {
         if (!gdm_session_settings_is_loaded (worker->priv->user_settings)) {
                 return;
