@@ -1101,6 +1101,9 @@ on_start_session_later (GdmGreeterServer *session,
 static void
 setup_server (GdmSimpleSlave *slave)
 {
+        /* Put cursor out of the way on first head */
+        gdm_slave_set_initial_cursor_position (GDM_SLAVE (slave));
+
         /* Set the busy cursor */
         gdm_slave_set_busy_cursor (GDM_SLAVE (slave));
 }
